@@ -9,7 +9,6 @@ __author__ = "Justin Forest"
 __email__ = "hex@umonkey.net"
 
 
-import appindicator
 import datetime
 import dateutil.parser
 import json
@@ -23,6 +22,12 @@ import pygtk
 pygtk.require("2.0")
 import gtk
 
+try:
+    import appindicator
+    HAVE_APPINDICATOR = True
+except ImportError:
+    HAVE_APPINDICATOR = False
+
 from taskindicator import database
 from taskindicator import properties
 from taskindicator import search
@@ -33,6 +38,10 @@ from taskindicator import util
 FREQUENCY = 1  # seconds
 
 
+def log(msg):
+    print(msg, file=sys.stderr)
+
+
 def get_program_path(command):
     for path in os.getenv("PATH").split(os.pathsep):
         full = os.path.join(path, command)
@@ -40,20 +49,35 @@ def get_program_path(command):
             return full
 
 
-class Checker(object):
-    """The indicator applet.  Displays the TaskWarrior icon and current
-    activity time, if any.  The pop-up menu can be used to start or stop
-    running tasks.
+class BaseIndicator(object):
+    def on_quit(self):
+        log("on_quit not handled")
+
+    def on_toggle(self):
+        log("on_toggle not handled")
+
+    def on_add_task(self):
+        log("on_add_task not handled")
+
+    def on_stop_all(self):
+        log("on_stop_all not handled")
+
+    def on_task_selected(self, task):
+        log("on_task_selected not handler, %s" % task)
+
+
+class UbuntuIndicator(BaseIndicator):
+    """
+    AppIndicator based icon
+
+    Used only if appindicator is available and running.
     """
     appname = "task-indicator"
     icon = "taskui"
     icon_attn = "taskui-active"
 
     def __init__(self):
-        self.toggle_lock = False
-
         self.task_items = []
-        self.database = database.Database(callback=self.on_tasks_changed)
 
         self.indicator = appindicator.Indicator(self.appname,
             self.icon, appindicator.CATEGORY_APPLICATION_STATUS)
@@ -70,23 +94,6 @@ class Checker(object):
         self.menu_setup()
         self.indicator.set_menu(self.menu)
 
-        self.dialog = properties.Dialog(callback=self.on_task_info_closed)
-        self.dialog.on_task_start = self.on_start_task
-        self.dialog.on_task_stop = self.on_stop_task
-
-        self.search_dialog = search.Dialog()
-        self.search_dialog.on_activate_task = self.on_search_callback
-
-        self.database.start_polling()
-
-    def on_start_task(self, task):
-        util.run_command(["task", task["uuid"], "start"])
-        self.update_status()
-
-    def on_stop_task(self, task):
-        util.run_command(["task", task["uuid"], "stop"])
-        self.update_status()
-
     def menu_setup(self):
         self.menu = gtk.Menu()
 
@@ -97,28 +104,30 @@ class Checker(object):
             self.menu.append(item)
             return item
 
-        self.add_task_item = add_item("Add new task...", self.on_add_task)
-        self.show_all_item = add_item("Show more...", self.on_show_all_tasks)
+        self.add_task_item = add_item("Add new task...",
+                                      lambda *args: self.on_add_task())
+
+        self.show_all_item = add_item("Show more...",
+                                      lambda *args: self.on_toggle())
+
         self.stop_item = add_item("Stop all running tasks", self.stop)
         if get_program_path("bugwarrior-pull"):
             self.bw_item = add_item("Pull tasks from outside",
                 self.on_pull_tasks)
-        self.quit_item = add_item("Quit", self.quit)
 
-    def menu_add_tasks(self):
-        print("Updating menu contents.", file=sys.stderr)
+        self.quit_item = add_item("Quit", lambda *args: self.on_quit())
+
+    def set_tasks(self, tasks):
+        log("Updating menu contents.")
 
         for item in self.menu.get_children():
             if item.get_data("is_dynamic"):
                 self.menu.remove(item)
         self.task_items = []
 
-        data = [t for t in self.database.get_tasks()
-            if t["status"] == "pending"]
-
-        for task in sorted(data, key=self.task_sort)[:10]:
+        for task in tasks[:10]:
             item = gtk.CheckMenuItem(self.format_menu_label(task),
-                use_underline=False)
+                                     use_underline=False)
             if task.get("start"):
                 item.set_active(True)
             item.connect("activate", self.on_task_toggle)
@@ -143,6 +152,161 @@ class Checker(object):
                 task["project"].split(".")[-1])
         return title
 
+    def set_idle(self):
+        self.indicator.set_label("Idle")
+        self.indicator.set_status(appindicator.STATUS_ACTIVE)
+        self.stop_item.hide()
+
+    def set_running(self, count, duration):
+        self.indicator.set_status(appindicator.STATUS_ATTENTION)
+        self.stop_item.show()
+
+        msg = "{0}/{1}".format(count, duration)
+        self.indicator.set_label(msg)
+
+    @classmethod
+    def is_available(cls):
+        if not HAVE_APPINDICATOR:
+            log("No appindicator package.")
+            return False  # not installed
+
+        user = os.getenv("USER")
+
+        p = subprocess.Popen(["pgrep", "-u", user, "indicator-applet"],
+            stdout=subprocess.PIPE)
+        out = p.communicate()[0]
+
+        if out.strip() == "":
+            log("User %s has no indicator-applet running." % user)
+            return False  # not running
+
+        return True
+
+
+class GtkIndicator(BaseIndicator):
+    def __init__(self):
+        self.icon = gtk.StatusIcon()
+        self.icon.set_from_icon_name("taskui")
+        self.icon.connect("activate",
+                          lambda *args: self.on_toggle())
+        self.icon.connect("popup-menu", self.on_menu)
+        self.icon.set_tooltip("TaskWarrior")
+
+        def add_item(label, handler):
+            item = gtk.MenuItem()
+            item.set_label(label)
+            item.connect("activate", lambda *args: handler())
+            item.show()
+            self.menu.append(item)
+            return item
+
+        self.menu = gtk.Menu()
+
+        self.task_items = []
+        for x in range(10):
+            item = gtk.CheckMenuItem()
+            item.set_label("task placeholder")
+            item.connect("activate",
+                         lambda item: self.on_task_selected(item.get_data("task")))
+            self.menu.append(item)
+            self.task_items.append(item)
+
+        self.separator = gtk.SeparatorMenuItem()
+        self.menu.append(self.separator)
+
+        add_item("Add new task...", self.on_add_task)
+        add_item("Search tasks...", self.on_toggle)
+        add_item("Stop all running tasks", self.on_stop_all)
+        add_item("Quit", self.on_quit)
+
+    def on_menu(self, icon, button, click_time):
+        self.menu.popup(None, None, gtk.status_icon_position_menu, button, click_time, self.icon)
+
+    def set_tasks(self, tasks):
+        for idx, item in enumerate(self.task_items):
+            if idx > len(tasks):
+                item.hide()
+            else:
+                task = tasks[idx]
+                desc = util.strip_description(task["description"])
+
+                item.set_label(desc)
+                item.set_active(task.is_active())
+                item.set_data("task", task)
+                item.show()
+
+        if tasks:
+            self.separator.show()
+        else:
+            self.separator.hide()
+
+    def set_idle(self):
+        self.icon.set_from_icon_name("taskui")
+        self.icon.set_tooltip("No running tasks")
+
+    def set_running(self, count, duration):
+        self.icon.set_from_icon_name("taskui-active")
+
+        if count == 1:
+            self.icon.set_tooltip("%s" % duration)
+        else:
+            self.icon.set_tooltip("%u tasks, %s" % (count, duration))
+
+
+class Checker(object):
+    """The indicator applet.  Displays the TaskWarrior icon and current
+    activity time, if any.  The pop-up menu can be used to start or stop
+    running tasks.
+    """
+
+    def __init__(self):
+        self.toggle_lock = False
+
+        self.setup_indicator()
+
+        self.database = database.Database(callback=self.on_tasks_changed)
+
+        self.dialog = properties.Dialog(callback=self.on_task_info_closed)
+        self.dialog.on_task_start = self.on_start_task
+        self.dialog.on_task_stop = self.on_stop_task
+
+        self.search_dialog = search.Dialog()
+        self.search_dialog.on_activate_task = self.on_search_callback
+
+        self.database.start_polling()
+
+    def setup_indicator(self):
+        if UbuntuIndicator.is_available():
+            log("AppIndicator is available, using it.")
+            self.indicator = UbuntuIndicator()
+        else:
+            log("AppIndicator is not available, using Gtk status icon.")
+            self.indicator = GtkIndicator()
+
+        self.indicator.on_add_task = self.on_add_task
+        self.indicator.on_toggle = self.on_toggle_search
+        self.indicator.on_stop_all = self.on_stop_all
+        self.indicator.on_quit = self.on_quit
+        self.indicator.on_task_selected = self.on_task_selected
+
+    def on_start_task(self, task):
+        util.run_command(["task", task["uuid"], "start"])
+        self.update_status()
+
+    def on_stop_task(self, task):
+        util.run_command(["task", task["uuid"], "stop"])
+        self.update_status()
+
+    def menu_add_tasks(self):
+        print("Updating menu contents.", file=sys.stderr)
+
+        tasks = filter(lambda t: t["status"] == "pending",
+                       self.database.get_tasks())
+
+        tasks = sorted(tasks, key=self.task_sort)
+
+        self.indicator.set_tasks(tasks)
+
     def task_sort(self, task):
         """Returns the data to sort tasks by."""
         tags = task.get("tags", [])
@@ -153,7 +317,7 @@ class Checker(object):
         return (-is_running, -is_pinned, is_endless, -pri,
             -float(task.get("urgency", 0)))
 
-    def on_add_task(self, widget):
+    def on_add_task(self):
         self.dialog.show_task({"uuid": None, "status": "pending",
             "description": "", "priority": "M"})
 
@@ -162,8 +326,11 @@ class Checker(object):
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE)
 
-    def on_show_all_tasks(self, widget):
-        self.search_dialog.show_all()
+    def on_toggle_search(self, *args):
+        if self.search_dialog.get_property("visible"):
+            self.search_dialog.hide()
+        else:
+            self.search_dialog.show_all()
 
     def on_search_callback(self, uuid):
         """Called when opening a task in the search window."""
@@ -173,17 +340,9 @@ class Checker(object):
             task = util.get_task_info(uuid)
         self.dialog.show_task(task)
 
-    def on_task_toggle(self, widget):
-        if self.toggle_lock:
-            return
-
-        self.toggle_lock = True
-        widget.set_active(not widget.get_active())
-
-        task = widget.get_data("task")
-        self.dialog.show_task(task)
-        self.toggle_lock = False
-        return True
+    def on_task_selected(self, task):
+        if task:
+            self.dialog.show_task(task)
 
     def on_task_info_closed(self, updates):
         """Updates the task when the task info window is closed.  Updates
@@ -224,7 +383,7 @@ class Checker(object):
         """Enters the main program loop"""
         self.on_timer()
 
-    def stop(self, widget):
+    def on_stop_all(self):
         """Stops running tasks"""
         for task in self.database.get_tasks():
             if "start" in task:
@@ -232,7 +391,7 @@ class Checker(object):
         self.stop_item.hide()
         self.update_status()
 
-    def quit(self, widget):
+    def on_quit(self):
         """Ends the applet"""
         sys.exit(0)
 
@@ -254,17 +413,11 @@ class Checker(object):
             if t["status"] == "pending" and "start" in t]
 
         if not tasks:
-            self.indicator.set_label("Idle")
-            self.indicator.set_status(appindicator.STATUS_ACTIVE)
-            self.stop_item.hide()
+            self.indicator.set_idle()
         else:
-            self.indicator.set_status(appindicator.STATUS_ATTENTION)
-            self.stop_item.show()
-
-            msg = "{0}/{1}".format(len(tasks),
-                self.format_duration(self.get_duration()))
-
-            self.indicator.set_label(msg)
+            count = len(tasks)
+            duration = self.format_duration(self.get_duration())
+            self.indicator.set_running(count, duration)
 
     def get_duration(self):
         duration = 0
@@ -287,5 +440,5 @@ def main():
 
     app = Checker()
     app.main()
-    app.search_dialog.show_all()
+    # app.search_dialog.show_all()
     gtk.main()
